@@ -65,6 +65,7 @@ QT_BEGIN_NAMESPACE
 class QQmlAbstractBinding;
 struct QQmlTypeCompiler;
 class QQmlInstantiationInterrupt;
+class QQmlIncubatorPrivate;
 
 struct QQmlObjectCreatorSharedState : public QSharedData
 {
@@ -80,17 +81,17 @@ struct QQmlObjectCreatorSharedState : public QSharedData
     QRecursionNode recursionNode;
 };
 
-class QQmlObjectCreator
+class Q_QML_PRIVATE_EXPORT QQmlObjectCreator
 {
     Q_DECLARE_TR_FUNCTIONS(QQmlObjectCreator)
 public:
-    QQmlObjectCreator(QQmlContextData *parentContext, QV4::CompiledData::CompilationUnit *compilationUnit, QQmlContextData *creationContext, void *activeVMEDataForRootContext = 0);
+    QQmlObjectCreator(QQmlContextData *parentContext, QV4::CompiledData::CompilationUnit *compilationUnit, QQmlContextData *creationContext, QQmlIncubatorPrivate  *incubator = 0);
     ~QQmlObjectCreator();
 
     QObject *create(int subComponentIndex = -1, QObject *parent = 0, QQmlInstantiationInterrupt *interrupt = 0);
-    bool populateDeferredProperties(QObject *instance);
+    bool populateDeferredProperties(QObject *instance, QQmlData::DeferredData *deferredData);
+    bool populateDeferredBinding(const QQmlProperty &qmlProperty, QQmlData::DeferredData *deferredData, const QV4::CompiledData::Binding *binding);
     QQmlContextData *finalize(QQmlInstantiationInterrupt &interrupt);
-    void cancel(QObject *object);
     void clear();
 
     QQmlComponentAttached **componentAttachment() const { return &sharedState->componentAttached; }
@@ -122,7 +123,8 @@ private:
 
     void registerObjectWithContextById(const QV4::CompiledData::Object *object, QObject *instance) const;
 
-    QV4::Heap::QmlContext *currentQmlContext();
+    inline QV4::QmlContext *currentQmlContext();
+    Q_NEVER_INLINE void createQmlContext();
 
     enum Phase {
         Startup,
@@ -143,7 +145,7 @@ private:
     const QQmlPropertyCacheVector *propertyCaches;
     QExplicitlySharedDataPointer<QQmlObjectCreatorSharedState> sharedState;
     bool topLevelCreator;
-    void *activeVMEDataForRootContext;
+    QQmlIncubatorPrivate *incubator;
 
     QObject *_qobject;
     QObject *_scopeObject;
@@ -172,6 +174,14 @@ private:
     QRecursionWatcher<QQmlObjectCreatorSharedState, &QQmlObjectCreatorSharedState::recursionNode> watcher;
 };
 
+QV4::QmlContext *QQmlObjectCreator::currentQmlContext()
+{
+    if (!_qmlContext->isManaged())
+        _qmlContext->setM(QV4::QmlContext::create(v4->rootContext(), context, _scopeObject));
+
+    return _qmlContext;
+}
+
 QT_END_NAMESPACE
 
 #endif // QQMLOBJECTCREATOR_P_H
